@@ -1,4 +1,5 @@
 import pytest
+from collections import OrderedDict
 from datetime import datetime
 
 from marshmallow import EXCLUDE, fields, INCLUDE, RAISE, Schema, validate
@@ -11,9 +12,9 @@ from .utils import get_schemas, build_ref
 
 
 class TestMarshmallowFieldToOpenAPI:
-    def test_fields_with_missing_load(self, openapi):
+    def test_fields_with_load_default_load(self, openapi):
         class MySchema(Schema):
-            field = fields.Str(default="foo", missing="bar")
+            field = fields.Str(dump_default="foo", load_default="bar")
 
         res = openapi.schema2parameters(MySchema, location="query")
         if openapi.openapi_version.major < 3:
@@ -102,6 +103,21 @@ class TestMarshmallowSchemaToModelDefinition:
         res = openapi.schema2jsonschema(BandSchema(partial=("drummer",)))
         assert res["required"] == ["bassist"]
 
+    @pytest.mark.parametrize("ordered_schema", (True, False))
+    def test_ordered(self, openapi, ordered_schema):
+        class BandSchema(Schema):
+            class Meta:
+                ordered = ordered_schema
+
+            drummer = fields.Str()
+            bassist = fields.Str()
+
+        res = openapi.schema2jsonschema(BandSchema)
+        assert isinstance(res["properties"], OrderedDict) == ordered_schema
+
+        res = openapi.schema2jsonschema(BandSchema())
+        assert isinstance(res["properties"], OrderedDict) == ordered_schema
+
     def test_no_required_fields(self, openapi):
         class BandSchema(Schema):
             drummer = fields.Str()
